package com.redhat.tomcat;

import org.apache.catalina.startup.BootstrapService;

public class Daemon {

    /**
     * The application main program.
     *
     * @param args Command line arguments
     */
    public static void main(String[] args) {

	// Process and remove -D options, if present.
	for (int i = 0; i < args.length; i++) {

	    if (!args[i].startsWith("-D"))
		continue;

	    int equal = args[i].indexOf('=');
	    if (equal == -1) {
		System.err.println(
		    "tomcat: option requires an argument" +
		    " of the form VAR=VALUE -- `-D'");
		System.err.println(
		    "Try `tomcat --help' for more information.");
		System.exit(1);
	    }

	    String name = args[i].substring(2, equal);
	    String val = args[i].substring(equal + 1);
	    System.setProperty(name, val);

	    String[] newargs = new String[args.length - 1];
	    for (int j = 0; j < i; j++)
		newargs[j] = args[j];
	    for (int j = i; j < newargs.length; j++)
		newargs[j] = args[j + 1];
	    args = newargs;

	    i--;

	}

	// Create and start the service
	BootstrapService service = new BootstrapService();
	try {
	    
	    service.init(args);
	    service.start();
	    daemon();
	    service.stop();
	    service.destroy();

	}
	catch (Exception e) {

	    e.printStackTrace(System.out);
	}

    }

    /**
     * Sit around and respond to the following signals:
     * <dl>
     *   <dt><code>SIGTERM</code><dd>Clean up and exit
     *   <dt><code>SIGINT</code><dd>Likewise
     *   <dt><code>SIGUSR1</code><dd>Recreate the log file (if in use)
     * </dl>
     */
    private native static void daemon();

    /**
     * Return the current log file serial number. This is bumped every
     * time a <code>SIGUSR1</code> is received. Our custom logger
     * class checks the serial number ever time it logs a message, and
     * will reopen its file if it has changed.
     */
    public native static int getLogSerial();
}
