package com.redhat.tomcat;

import java.lang.reflect.Method;
import java.sql.Timestamp;
import org.apache.catalina.logger.FileLogger;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class Logger extends FileLogger {

    /**
     * The serial number for the currently open log file, or -1 if
     * there is no open log file.
     */
    private int serial = -1;

    /**
     * The descriptive information about this implementation.
     */
    protected static String info = "com.redhat.tomcat.Logger/1.0";

    /**
     * The method used to obtain the current serial number.
     */
    private static Method getLogSerialImpl = getGetLogSerial();

    /**
     * Return the log file name.
     */
    public String getFilename() {

	return getPrefix() + getSuffix();

    }

    /**
     * Set the log file name.
     */
    public void setFilename(String filename) {

        int dot = filename.lastIndexOf('.');
        if (dot >= 0) {
	    setPrefix(filename.substring(0, dot));
	    setSuffix(filename.substring(dot));
	} else {
	    setPrefix(filename);
	    setSuffix("");
	}

    }

    /**
     * Return the method used to obtain the current serial number.
     */
    private static Method getGetLogSerial() {

	Class klass = null;
	try {
	    klass = Class.forName("com.redhat.tomcat.Daemon");
	} catch (ClassNotFoundException e) {
	    // How should a logger log its errors?  It's all gone Zen ;)
	    Log log = LogFactory.getLog(Logger.class);
	    log.warn(Logger.class.getName() +
		     " used outside of daemon context;");
	    log.warn("Logs will not be rotated.");
	    return null;
	}

	Method method = null;
	try {
	    method = klass.getMethod("getLogSerial", null);
	} catch (NoSuchMethodException e) {
	    Log log = LogFactory.getLog(Logger.class);
	    log.error("Can't find Daemon.getLogSerial()", e);
	    log.warn("Logs will not be rotated.");
	    return null;
	}

	return method;
    }

    /**
     * Return the current serial number, or -1 if not in the daemon.
     */
    private static int getLogSerial() {

	if (getLogSerialImpl == null)
	    return -1;

	Object serial = null;
	try {
	    serial = getLogSerialImpl.invoke(null, new Object[] {});
	} catch (Exception e) {
	    Log log = LogFactory.getLog(Logger.class);
	    log.error("Error invoking Daemon.getLogSerial()", e);
	    log.warn("Logs will not be rotated.");
	    getLogSerialImpl = null; // don't fill their logs
	    return -1;
	}

	return ((Integer) serial).intValue();
    }

    /**
     * Writes the specified message to a servlet log file, usually an event
     * log.  The name and type of the servlet log is specific to the
     * servlet container.
     *
     * @param msg A <code>String</code> specifying the message to be written
     *  to the log file
     */
    public void log(String msg) {

        // If the serial number has changed, reopen the file
	int current_serial = getLogSerial();
        if (current_serial != serial) {
            synchronized (this) {
                if (current_serial != serial) {
                    close();
                    open();
		    serial = current_serial;
                }
            }
        }

        // Log this message, timestamped if necessary
        if (writer != null) {
            if (timestamp) {
		Timestamp ts = new Timestamp(System.currentTimeMillis());
		String tsString = ts.toString().substring(0, 19);
                writer.println(tsString + " " + msg);
            } else {
                writer.println(msg);
            }
        }

    }

}
